// engine.h
/*
 *  Copyright (c) 2010 Leigh Johnston.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of Leigh Johnston nor the names of any
 *       other contributors to this software may be used to endorse or
 *       promote products derived from this software without specific prior
 *       written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 *  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef DDLC_ENGINE
#define DDLC_ENGINE

#include <lib/threadable.h>
#include <list>

class engine : lib::threadable
{
public:
	// construction
	engine();
	~engine();

public:
	// operations
	void add_file(const std::string& aFile);
	unsigned int progress() const;
	unsigned int file_progress() const;
	void results(unsigned long& aFileCount, unsigned long& aFailures, unsigned long& aCharacterCount, unsigned long& aLineCount) const;
	void reset();
	bool analyzing() const;
	unsigned long failures() const { return iFailures; }

private:
	// implementation
	bool any_left();
	bool analyze_file(const std::string& aFile, unsigned long& aLineCount, unsigned long& aCharacterCount);
	void analyze_files();
	// from lib::threadable
	virtual void task() { analyze_files(); }
	virtual bool soft_abort() const { return true; }

private:
	// attributes
	typedef std::list<std::string> file_list;
	file_list iFiles;
	file_list::iterator iCurrentFile;
	unsigned long iCurrentFileSize;
	unsigned long iCurrentFileRead;
	unsigned long iFileCount;
	unsigned long iFailures;
	unsigned long iCharacterCount;
	unsigned long iLineCount;
};

#endif // DDLC_ENGINE
