// detail_memory.h
/*
 *  Copyright (c) 2010 Leigh Johnston.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of Leigh Johnston nor the names of any
 *       other contributors to this software may be used to endorse or
 *       promote products derived from this software without specific prior
 *       written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 *  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef LIB_DETAIL_MEMORY
#define LIB_DETAIL_MEMORY

#include <cstring>
#include <type_traits>

namespace lib 
{
	namespace detail
	{
		template <typename T> inline
		void construct(void* mem, const T& object)
		{
			new (mem) T(object);
		}

		template <typename InIter, typename OutIter> inline
		OutIter uninitialized_copy_dispatch(InIter first, InIter last, OutIter result, std::tr1::false_type)
		{
			while (first != last)
				detail::construct(static_cast<void*>(&*result++), *first++);
			return result;
		}

		template <typename T> inline
		T* uninitialized_copy_dispatch(const T* first, const T* last, T* result, std::tr1::true_type)
		{
			memcpy(result, first, (last-first) * sizeof(T));
			result += (last-first);
			return result;
		}

		template <typename InIter, typename OutIter, typename T> inline
		OutIter uninitialized_copy(InIter first, InIter last, OutIter result, const T&)
		{
			return uninitialized_copy_dispatch(first, last, result, std::tr1::is_scalar<T>::type());
		}
	}
}

#endif // LIB_DETAIL_MEMORY
