// lockable.h v1.2.3
/*
 *  Copyright (c) 2010 Leigh Johnston.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of Leigh Johnston nor the names of any
 *       other contributors to this software may be used to endorse or
 *       promote products derived from this software without specific prior
 *       written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 *  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef LIB_LOCKABLE
#define LIB_LOCKABLE

#include <windows.h>
#include <intrin.h>
#pragma intrinsic(_ReadBarrier)
#pragma intrinsic(_WriteBarrier)
#pragma intrinsic(_ReadWriteBarrier)

namespace lib
{
	class lockable
	{
		// construction
	public:
		lockable();
		lockable(const lockable& aOther);
		virtual ~lockable();
		// operations
	public:
		void lock() const;
		void unlock() const;
		// attributes
	private:
		mutable CRITICAL_SECTION iCriticalSection;
	};

	class lock
	{
		// construction
	public:
		lock(const lockable& aLockable) : iLockable(aLockable) { iLockable.lock(); }
		~lock() { iLockable.unlock(); }
	private:
		lock& operator=(const lock&);
		// attributes
	private:
		const lockable& iLockable;
	};

	inline void memory_barrier_acquire()
	{
		_ReadBarrier();
	}

	inline void memory_barrier_acquire_dependant()
	{
		_ReadBarrier();
	}

	inline void memory_barrier_release()
	{
		_WriteBarrier();
	}

	inline void memory_barrier()
	{
		_ReadWriteBarrier();
	}
}

#endif //LIB_LOCKABLE
