// plugin.h
/*
 *  Copyright (c) 2010 Leigh Johnston.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of Leigh Johnston nor the names of any
 *       other contributors to this software may be used to endorse or
 *       promote products derived from this software without specific prior
 *       written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 *  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef LIB_PLUGIN
#define LIB_PLUGIN

namespace lib
{
	namespace plugin
	{
		struct object  
		{
			virtual void add_ref() const = 0;
			virtual void release() const = 0;
		};
		struct resource : object
		{
			virtual bool valid() const = 0;
		};
		struct string : object
		{
			typedef char* pointer;
			typedef const char* const_pointer;
			typedef wchar_t* wide_pointer;
			typedef const wchar_t* const_wide_pointer;
			typedef pointer utf8_pointer;
			typedef const_pointer const_utf8_pointer;
			typedef wide_pointer utf16_pointer;
			typedef const_wide_pointer const_utf16_pointer;
			virtual const_utf8_pointer as_utf8() const = 0;
			virtual const_utf16_pointer as_utf16() const = 0;
			virtual string& operator=(const_utf8_pointer aUtf8String) = 0;
			virtual string& operator=(const_utf16_pointer aUtf16String) = 0;
		};
		struct plugin_host : object
		{
			virtual void create_string(string*& aString) const = 0;
			virtual void create_string(string*& aString, string::const_pointer aContents) const = 0;
			virtual void create_string(string*& aString, string::const_wide_pointer aContents) const = 0;
			virtual void root_path(string*& aString) const = 0;
			virtual void add_menu_item(unsigned int aMenuId, unsigned int aCommandId, const string& aText) = 0;
			virtual void remove_menu_item(unsigned int aMenuId, unsigned int aCommandId) = 0;
		};
		struct plugin_info
		{
			const char* iName;
			const char* iDescription;
			void* iIcon;
		};
		struct plugin : object
		{
			// the following functions are run in the context of the main host thread
			virtual void info(plugin_info& aInfo) const = 0;
			virtual void properties() = 0;
			// the following functions are run in the context of a plugin thread
			virtual void start() = 0;
			virtual void stop() = 0;
			virtual void menu_clicked(unsigned int aMenuId, unsigned int aCommandId) = 0;
		};
	}
}

#endif //LIB_PLUGIN
